@@ -970,7 +970,8 @@ class Nag_Api extends Horde_Registry_Api
      * @param string $tasklist_id  The tasklist that contains the task.
      *
      * @return boolean|string  True if the task has been toggled, a due date if
-     *                         there are still incomplete recurrences.
+     *                         there are still incomplete recurrences, otherwise
+     *                         false.
      */
     public function toggleCompletion($task_id, $tasklist_id)
     {
@@ -985,7 +986,12 @@ class Nag_Api extends Horde_Registry_Api
         }
         $task = Nag::getTask($tasklist_id, $task_id);
         $completed = $task->completed;
-        $task->toggleComplete();
+        try {
+            $task->toggleComplete();
+        } catch (Nag_Exception $e) {
+            Horde::log($e->getMessage(), 'DEBUG');
+            return false;
+        }
         $task->save();
         $due = $task->getNextDue();
         if ($task->completed == $completed) {
@@ -1334,7 +1340,7 @@ class Nag_Api extends Horde_Registry_Api
         // List incomplete tasks.
         $tasks = Nag::listTasks(array(
             'tasklists' => $categories,
-            'completed' => Nag::VIEW_INCOMPLETE,
+            'completed' => Nag::VIEW_FUTURE_INCOMPLETE,
             'include_history' => false)
         );
 
