@@ -160,7 +160,7 @@ abstract class Nag_Driver
         $result = Nag::sendNotification('add', $task);
 
         /* Add an alarm if necessary. */
-        if (!empty($task->alarm) &&
+        if (!empty($task->due) && !empty($task->alarm) &&
             ($alarm = $task->toAlarm())) {
             $GLOBALS['injector']->getInstance('Horde_Alarm')->set($alarm);
         }
@@ -289,12 +289,20 @@ abstract class Nag_Driver
                 }
                 $log_tasklist = $properties['tasklist'];
             }
+            $task->loadChildren();
+            if ($task->hasSubTasks()) {
+                foreach ($task->children as $child_task) {
+                    $child_task = $child_task->toHash();
+                    $child_task['tasklist'] = $properties['tasklist'];
+                    $this->modify($child_task['task_id'], $child_task);
+                }
+            }
         }
 
         /* Update alarm if necessary. */
         $horde_alarm = $GLOBALS['injector']->getInstance('Horde_Alarm');
         if ((isset($properties['alarm']) && empty($properties['alarm'])) ||
-            !empty($properties['completed'])) {
+            !empty($properties['completed']) || empty($task->due)) {
             $horde_alarm->delete($task->uid);
         } else {
             $task = $this->get($taskId);
@@ -378,8 +386,21 @@ abstract class Nag_Driver
     {
         /* Get the task's details for use later. */
         $task = $this->get($taskId);
+        $task->loadChildren();
+        if ($task->hasSubTasks()) {
+            throw new Nag_Exception(_("Sub tasks exist, delete them first"));
+        }
+
         $delete = $this->_delete($taskId);
 
+        /* Remove tags */
+        $task->tags = array();
+        $this->_updateTags($task->toHash());
+
+        /* Tell content we removed the object */
+        $GLOBALS['injector']->getInstance('Content_Objects_Manager')
+            ->delete(array($task->uid), 'task');
+
         /* Log the deletion of this item in the history log. */
         if (!empty($task->uid)) {
             try {
@@ -424,14 +445,21 @@ abstract class Nag_Driver
     {
         $ids = $this->_deleteAll();
 
-        // Update History.
+        // Update History and Tagger
         $history = $GLOBALS['injector']->getInstance('Horde_History');
         try {
-            foreach ($ids as $id) {
+            foreach ($ids as $uid) {
                 $history->log(
-                    'nag:' . $this->_tasklist . ':' . $id,
+                    'nag:' . $this->_tasklist . ':' . $uid,
                     array('action' => 'delete'),
                     true);
+
+                $GLOBALS['injector']->getInstance('Nag_Tagger')
+                    ->replaceTags($uid, array(), $GLOBALS['registry']->getAuth(), 'task');
+
+                /* Tell content we removed the object */
+                $GLOBALS['injector']->getInstance('Content_Objects_Manager')
+                    ->delete(array($uid), 'task');
             }
         } catch (Exception $e) {
             Horde::log($e, 'ERR');
