@@ -2,7 +2,7 @@
 /**
  * Nag driver classes for the Kolab IMAP server.
  *
- * Copyright 2004-2015 Horde LLC (http://www.horde.org/)
+ * Copyright 2004-2016 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -94,6 +94,45 @@ class Nag_Driver_Kolab extends Nag_Driver
         return new Nag_Task($this, $nag_task);
     }
 
+    /**
+     * Detect if a string is already Horde_Url::uriB64Encode()'ed or not.
+     *
+     * @param string $s  The string to test.
+     *
+     * @return  boolean  True if $s was base64 encoded.
+     * @todo  This will be removed in Horde 6.
+     */
+    public function _isBase64EncodedUid($s)
+    {
+         // Redo stuff possibly done by Horde_Url::uriB64Encode()
+        $data = str_replace(array('-', '_'), array('+', '/'), $s);
+        $mod4 = strlen($data) % 4;
+        if ($mod4) {
+            $data .= substr('====', $mod4);
+        }
+        if (!preg_match('/^[a-zA-Z0-9\/\r\n+]*={0,2}$/', $data)) {
+            return false;
+        }
+
+         // Decode the string in strict mode and check the results
+        $decoded = base64_decode($data, true);
+        if ($decoded === false) {
+            return false;
+        }
+
+        // Check if the decoded string resembles a uid hash...
+        if (!preg_match('/^[a-zA-Z0-9-]+$/', $decoded)) {
+            return false;
+        }
+
+         // Encode the string again for a final sanity check.
+        if (base64_encode($decoded) != $data) {
+            return false;
+        }
+
+        return true;
+    }
+
     /**
      * Build a task based a data array
      *
@@ -103,13 +142,22 @@ class Nag_Driver_Kolab extends Nag_Driver
      */
     protected function _buildTask($task)
     {
+         // This decoding of parent ID hashes is required because of a
+         // previous bug in Nag_Driver_Kolab.
+         // See Bug: 14197
+        $parent_uid = $task['parent'];
+        if (!empty($parent_uid) && $this->_isBase64EncodedUid($parent_uid)) {
+            $parent_uid = Horde_Url::uriB64Decode($parent_uid);
+        }
+
         $result = array(
             'task_id' => Horde_Url::uriB64Encode($task['uid']),
             'uid' => $task['uid'],
             'name' => $task['summary'],
             'desc' => $task['body'],
             'priority' => $task['priority'],
-            'parent' => $task['parent'],
+            // parent is keyed to internal id, not UID.
+            'parent' => Horde_Url::uriB64Encode($parent_uid),
             'alarm' => $task['alarm'],
             'completed' => !empty($task['completed']),
             'completed_date' => $task['completed_date'],
@@ -204,7 +252,7 @@ class Nag_Driver_Kolab extends Nag_Driver
     protected function _add(array $task)
     {
         $object = $this->_getObject($task);
-        $object['uid'] = $this->_getData()->generateUid(); 
+        $object['uid'] = $this->_getData()->generateUid();
         try {
             $this->_getData()->create($object);
             $this->_addTags($task);
@@ -287,7 +335,8 @@ class Nag_Driver_Kolab extends Nag_Driver
             'body' => $task['desc'],
             //@todo: Match Horde/Kolab priority values
             'priority' => $task['priority'],
-            'parent' => $task['parent'],
+            // Kolab's parent field is UID, Nag's parent is id.
+            'parent' => Horde_Url::uriB64Decode($task['parent']),
         );
         if (!empty($task['start'])) {
             $object['start-date'] = new DateTime('@' . $task['start']);
@@ -415,7 +464,7 @@ class Nag_Driver_Kolab extends Nag_Driver
             switch ($completed) {
             case Nag::VIEW_INCOMPLETE:
                 if ($complete) {
-                    continue;
+                    continue 2;
                 }
                 if ($start && $t->recurs() &&
                     ($completions = $t->recurrence->getCompletions())) {
@@ -431,18 +480,18 @@ class Nag_Driver_Kolab extends Nag_Driver
                         ->nextRecurrence($lastCompletion)
                         ->timestamp();
                     if ($start > $_SERVER['REQUEST_TIME']) {
-                        continue;
+                        continue 2;
                     }
                 }
                 break;
             case Nag::VIEW_COMPLETE:
                 if (!$complete) {
-                    continue;
+                    continue 2;
                 }
                 break;
             case Nag::VIEW_FUTURE:
                 if ($complete || $start == 0) {
-                    continue;
+                    continue 2;
                 }
                 if ($start && $t->recurs() &&
                     ($completions = $t->recurrence->getCompletions())) {
@@ -458,13 +507,13 @@ class Nag_Driver_Kolab extends Nag_Driver
                         ->nextRecurrence($lastCompletion)
                         ->timestamp();
                     if ($start < $_SERVER['REQUEST_TIME']) {
-                        continue;
+                        continue 2;
                     }
                 }
                 break;
             case Nag::VIEW_FUTURE_INCOMPLETE:
                 if ($complete) {
-                    continue;
+                    continue 2;
                 }
                 break;
             }
@@ -518,11 +567,15 @@ class Nag_Driver_Kolab extends Nag_Driver
     /**
      * Retrieves sub-tasks from the database.
      *
-     * @param string $parentId  The parent id for the sub-tasks to retrieve.
+     * @param string $parentId          The parent id for the sub-tasks to
+     *                                  retrieve.
+     * @param boolean $include_history  Include created/modified info? Not
+     *                                  currently honored.
      *
      * @return array  List of sub-tasks.
+     * @throws Nag_Exception
      */
-    public function getChildren($parentId)
+    public function getChildren($parentId, $include_history = true)
     {
         $task_list = $this->_getData()->getObjects();
         if (empty($task_list)) {
